/*
    Authorize.net Legend (this is for the old school "transact.dll" call implimented before XML and JSON standards rolled out by Authorize.net)
		  LEGEND:
		  More at: http://www.authorize.net/support/merchant/Transaction_Response/Transaction_Response.htm
		  [0] = Overall status of transaction.  1 = approved, 2 = declined, 3 = error, 4 = held for review
		  [1] = Response sub-code.  Code used by the payment gateway for internal transaction tracking (AIM only)
		  [2] = Response reason code.  More details about the result of the transaction.  1 = approved, 2 = declined, 3 = error, 4 = held for review
		  [3] = Response text.  Brief description of the result of the transaction request
		  [4] = 6-digit authorization/approval code
		  [5] = AVS code (address verification service).  P = AVS not applicable for this transaction
		  [6] = Transaction ID for the request.  NOTE: must be used for credit, prior authorization and capture, void transactions
		  [7] = Merchant invoice number assigned to the transaction up to 20 alpha-numeric characters
		  [8] = Merchant description of the transaction up to 255 alpha-numeric characters
		  [9] = Amount of the transaction up to 15 digits
		  [10] = Payment method.  Either CC or ECHECK
		  [11] = Transaction type: AUTH_CAPTURE, AUTH_ONLY, CAPTURE_ONLY, CREDIT, PRIOR_AUTH_CAPTURE, VOID
		  [12] = Customer ID.  Merchant assigned customer ID up to 20 alpha-numberic characters
		  [32] = Tax.  Merchant assigned tax amount; numeric
		  [33] = Duty.  Merchant assigned duty amount; numeric
		  [34] = Frieght.  Merchant assigned frieght amount; numeric
		  [35] = Tax Exempt.  Merchant assigned tax exemption; either TRUE, T, YES, Y, 1 or FALSE, F, NO, N, 0
		  [36] = Merchant purchase order number up to 25 alpha-numeric characters
		  [37] = MD5 Hash.  Gateway generated MD5 hash value that may be used to authenticate the transaction response
		  [38] = CCV Response code.  M = match, N = no match, P = not processed, S = should have been present, U = issuer unable to process request
		  [39] = Cardholder authentication verification response
		  [50] = Truncated CC number in format like "XXXX0012" where last 4 of CC are present
		  [51] = CC Type such as "Discover"
*/
var https = require("https");

exports.handler = function(event, context) {
/* Define date information used to create expiration date for test CC */
var date = new Date();
var currentYear = date.getFullYear();
var currentMonth = date.getMonth() + 1;
if (currentMonth < 10) { currentMonth = "0" + currentMonth; }

/* Define Authorize.Net Connection */
var authnetValidateCert = "false";	/* In production this should be set to true to validate 3rd party certificate */
var authnetHost = "secure.authorize.net";
var authnetPort = 443;
var authnetPath = "/gateway/transact.dll";
var authnetMethod = "POST";
var authnetContentType = "application/x-www-form-urlencoded";
var authnetResponseData = [];

/* Define Authorize.Net Gateway credentials */
var gateway_login = "gatewaylogindetails";
var gateway_key = "gatewaykeydetails";

/* Define user purchase details */
var isTestTransaction = 1;  /* 1 = test transaction, 0 = not test transaction */
var amount = "4.49";
var ccnum = "6011000000000012"; /* Test CC */
var cvv = "330";    /* Test CVV */
var expdate = currentMonth + "/" + currentYear; /* Today's date in MM/YYYY format for testing */

/* Define request body */
var postfieldsPayload = "";
	postfieldsPayload = "x_version=3.1";
	postfieldsPayload = postfieldsPayload + "&x_delim_data=true";
	postfieldsPayload = postfieldsPayload + "&x_relay_response=false";
	postfieldsPayload = postfieldsPayload + "&x_login=" + gateway_login;
	postfieldsPayload = postfieldsPayload + "&x_tran_key=" + gateway_key;
	postfieldsPayload = postfieldsPayload + "&x_amount=" + amount;
	postfieldsPayload = postfieldsPayload + "&x_duplicate_window=10";
	postfieldsPayload = postfieldsPayload + "&x_card_num=" + ccnum;
	postfieldsPayload = postfieldsPayload + "&x_exp_date=" + expdate;
	postfieldsPayload = postfieldsPayload + "&x_card_code=" + cvv;
	postfieldsPayload = postfieldsPayload + "&x_type=AUTH_CAPTURE";
	if (isTestTransaction === 1) {
	    postfieldsPayload = postfieldsPayload + "&x_test_request=TRUE";
	}

/* Define request parameters */
var curlOptions = {
	host: authnetHost,
	port: authnetPort,
	path: authnetPath,
	rejectUnauthorized: authnetValidateCert,
	method: authnetMethod,
	headers: {
		'Content-Type': authnetContentType,
		'Content-Length': Buffer.byteLength(postfieldsPayload)
	}
};

/* Perform request */
var authnetRequest = https.request(curlOptions, function(result) {
	var responseData = "";
	/* Error */
	result.on('error', function(e) {
		context.fail('error: ' + e.message);
	});

	/* Collect Response */
	result.on('data', function(chunk)  {
		responseData += chunk;
	});

	/* Req / Res End. */
	result.on('end', function() {
	    /* Parse response data. */
	    authnetResponseData = responseData.split(",");
	    var transactionStatus = authnetResponseData[0]; /* 1 = approved, 2 = declined, 3 = error, 4 = held for review */
	    var transactionText = authnetResponseData[3];   /* Response text.  Brief description of the result of the transaction request */
	    var transactionId = authnetResponseData[6];     /* Transaction ID for the request */
	    /* Generate Response */
		context.done(
			null,
			{
			"transactionStatus": transactionStatus,
			"transactionText": transactionText,
			"transactionId": transactionId
			}
		);
	});
});

/* Call the https request to run the transaction and continue with other logic tasks. */
authnetRequest.write(postfieldsPayload);
authnetRequest.end(); /* Required "end" statement by Node.js standard */

};
